#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2016-04-03
#
# 2016-04-03: rewrite of the original cmk 1.x check by Th.L.
# 2016-05-24: added admin/operational state, fix for missing ap_name parameter
# 2016-07-01: added long output
# 2016-07-06: fixed: missing ap
# 2018-01-05: cosmetic changes, added metrics/cisco_wlc.py
# 2018-05-23: changed scan function, changed return values to yield
#             code cleanup, removed unused oid's
# 2018-07-26: added code for incomplete ap data (missing lwap, data)
# 2018-08-03: changed cdp info: short interface name, dropped neighbor domain name, added S/N and Model to longoutput
# 2018-09-14: fixed issue with missing AP, fixed issue with SSH check
# 2021-07-16: rewrite for CMK 2.0
# 2021-07-17: added basic support for IOS-XE based WLAN controllers (C9800 family), without most of perfdata
#             fixed _render_mac_address/_render_ip_address, code cleanup, removed WIPS check
#             added lwap_uptime/time taken to join. changed perfdata to 1/s
# 2021-07-29: fix missing data from discovery (inv_ap_info)
# 2022-03-07: fixed handling of missing AP (THX to andreas[dot]doehler[at]gmail[dot]com)
# 2022-04-01: added RADIO MAC address
# 2022-05-31: removed "Software" from snmp detect function
# 2022-10-23: fixed warning on upgrade "non-empty params vanished" for inv_ap_info
# 2023-02-21: added cluster function (backport from check_mk) THX to roger[dot]ellenberger[at]wagner[dot]ch
# 2023-02-21: moved gui files from ~/local/share/check_mk/... to ~/local/lib/check_mk...
# 2023-06-07: fixed ValueError (not enough values to unpack (expected 3, got 1)) in parse function (string_table)

from time import time
from dataclasses import dataclass
from typing import Optional, List, Dict, Mapping, Any, Union
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    SNMPTree,
    contains,
    render,
    check_levels,
    OIDEnd,
    Result,
    State,
    Metric,
    any_of,
    get_rate,
    GetRateError,
    get_value_store,
    IgnoreResultsError,
)


@dataclass
class CdpInfo:
    cdp_neigh_name: str
    cdp_neigh_address: str
    cdp_neigh_interface: str
    cdp_neigh_platform: str
    cdp_neigh_duplex: str


@dataclass
class LwapInfo:
    ap_uptime: Optional[int] = None
    ap_lwappuptime: Optional[int] = None
    ap_lwappjointakentime: Optional[int] = None
    ap_encryptionenable: Optional[int] = None
    ap_powerstatus: Optional[int] = None
    ap_telnetenable: Optional[int] = None
    ap_sshenable: Optional[int] = None
    ap_encryptionsupported: Optional[int] = None
    ap_dataencryptionstatus: Optional[int] = None
    ap_adminstatus: Optional[int] = None

    ap_assoccount: Optional[int] = None
    ap_associatedclientcount: Optional[int] = None
    ap_maxclientlimitnumbertrap: Optional[int] = None
    ap_reassocsucccount: Optional[int] = None
    ap_reassocfailcount: Optional[int] = None
    ap_activeclientcount: Optional[int] = None
    ap_assocfailcountforrssilow: Optional[int] = None
    ap_assocfailtimes: Optional[int] = None
    ap_echorequestcount: Optional[int] = None
    ap_echoresponselosscount: Optional[int] = None
    

@dataclass
class ApInfo:
    ap_location: str
    ap_model: str
    ap_serialnumber: str
    ap_ipaddress: str
    ap_ethernetmacaddress: str
    ap_radio_mac: str
    ap_operationstatus: Optional[int] = None


@dataclass
class Ap:
    ap_info: ApInfo
    cdp_info: Optional[CdpInfo] = None
    lwap_info: Optional[LwapInfo] = None


@dataclass
class InvApInfo:
    ap_location: str
    ap_model: str
    ap_serialnumber: str
    ap_ipaddress: str
    ap_ethernetmacaddress: str
    cdp_neigh_name: Optional[str] = None
    cdp_neigh_address: Optional[str] = None
    cdp_neigh_interface: Optional[str] = None
    cdp_neigh_platform: Optional[str] = None


Section = Dict[str, Ap]

_operationasl_state = {
    1: 'associated',
    2: 'disassociating',
    3: 'downloading',
}

_power_status = {
    0: 'N/A',
    1: 'low',
    2: '15.4W',
    3: '16.8W',
    4: 'full',
    5: 'external',
    6: 'mixedmode',
}

_cdp_duplex = {
    '0': 'N/A',
    '1': 'unknown',
    '2': 'full duplex',
    '3': 'half duplex',
}

_cdp_speed = {
    '0': 'N/A',
    '1': 'none',
    '2': '10Mbps',
    '3': '100Mbps',
    '4': '1000Mbps',
    '5': 'auto',
}

_interface_displayhints = {
    'ethernet': 'eth',
    'fastethernet': 'Fa',
    'gigabitethernet': 'Gi',
    'tengigabitethernet': 'Te',
    'fortygigabitethernet': 'Fo',
    'hundredgigabitethernet': 'Hu',
    'port-channel': 'Po',
    'tunnel': 'Tu',
    'loopback': 'Lo',
    'cellular': 'Cel',
    'vlan': 'Vlan',
    'management': 'Ma',
}

_map_states = {
    "1": (State.OK, "online"),
    "2": (State.CRIT, "critical"),
    "3": (State.WARN, "warning"),
}


def _get_short_if_name(ifname: str) -> str:
    """
    Return short interface name from long interface name.
    ifname: is the long interface name
    :type ifname: str
    """
    for ifname_prefix in _interface_displayhints.keys():
        if ifname.lower().startswith(ifname_prefix.lower()):
            ifname_short = _interface_displayhints[ifname_prefix]
            return ifname.lower().replace(ifname_prefix.lower(), ifname_short, 1)
    return ifname


def _render_mac_address(bytestring):
    macaddress = bytestring
    bytestring = bytestring.replace('|4|', ' ').replace('|4.', ' ').replace('"', '').strip(' ').strip('.')\
        .replace(' ', ':').replace('.', ':')

    if len(bytestring) == 17:
        return bytestring
    else:
        return ':'.join(['%02s' % hex(ord(m))[2:] for m in macaddress]).replace(' ', '0').upper()


def _render_ip_address(bytestring):
    if len(bytestring) == 4:
        return '.'.join(['%s' % ord(m) for m in bytestring])
    else:
        bytestring_clean = bytestring.replace('"', '').replace('|4|', '').replace('.', ' ').strip().split(' ')
        try:
            return '.'.join(['%s' % int(m, 16) for m in bytestring_clean])
        except ValueError:
            return bytestring


def _get_cdpdata(ap_name, cdpdata) -> Optional[CdpInfo]:
    for cdpentry in cdpdata:
        cdp_ap_name, cdp_neigh_name, cdp_neigh_address_type, cdp_neigh_address, cdp_neigh_interface, \
        cdp_neigh_platform, cdp_neigh_duplex = cdpentry
        if cdp_ap_name == ap_name:
            if int(cdp_neigh_address_type) == 1:
                cdp_neigh_address = _render_ip_address(cdp_neigh_address)
            return CdpInfo(
                cdp_neigh_name=cdp_neigh_name.split('.')[0],
                cdp_neigh_address=cdp_neigh_address,
                cdp_neigh_interface=_get_short_if_name(cdp_neigh_interface),
                cdp_neigh_platform=cdp_neigh_platform,
                cdp_neigh_duplex=cdp_neigh_duplex,
            )

    return None


def _get_lwapap_info(oid_end, lwapap) -> Optional[LwapInfo]:
    for lwapapentry in lwapap:
        lwap_oid_end, ap_uptime, ap_lwappuptime, ap_lwappjointakentime, ap_encryptionenable, ap_powerstatus, \
        ap_telnetenable, ap_sshenable, ap_encryptionsupported, ap_dataencryptionstatus, ap_adminstatus, ap_assoccount, \
        ap_associatedclientcount, ap_maxclientlimitnumbertrap, ap_reassocsucccount, ap_reassocfailcount, \
        ap_activeclientcount, ap_assocfailcountforrssilow, ap_assocfailtimes, ap_echorequestcount, \
        ap_echoresponselosscount = lwapapentry

        if lwap_oid_end == oid_end:
            return LwapInfo(
                ap_uptime=int(ap_uptime) if ap_uptime.isdigit() else None,
                ap_lwappuptime=int(ap_lwappuptime) if ap_lwappuptime.isdigit() else None,
                ap_lwappjointakentime=int(ap_lwappjointakentime) if ap_lwappjointakentime.isdigit() else None,
                ap_encryptionenable=int(ap_encryptionenable) if ap_encryptionenable.isdigit() else None,
                ap_powerstatus=int(ap_powerstatus) if ap_powerstatus.isdigit() else None,
                ap_telnetenable=int(ap_telnetenable) if ap_telnetenable.isdigit() else None,
                ap_sshenable=int(ap_sshenable) if ap_sshenable.isdigit() else None,
                ap_encryptionsupported=int(ap_encryptionsupported) if ap_encryptionsupported.isdigit() else None,
                ap_dataencryptionstatus=int(ap_dataencryptionstatus) if ap_dataencryptionstatus.isdigit() else None,
                ap_adminstatus=int(ap_adminstatus) if ap_adminstatus.isdigit() else None,

                ap_assoccount=int(ap_assoccount) if ap_assoccount.isdigit() else None,
                ap_associatedclientcount=int(ap_associatedclientcount) if ap_associatedclientcount.isdigit() else None,
                ap_maxclientlimitnumbertrap=int(ap_maxclientlimitnumbertrap) if ap_maxclientlimitnumbertrap.isdigit() else None,
                ap_reassocsucccount=int(ap_reassocsucccount) if ap_reassocsucccount.isdigit() else None,
                ap_reassocfailcount=int(ap_reassocfailcount) if ap_reassocfailcount.isdigit() else None,
                ap_activeclientcount=int(ap_activeclientcount) if ap_activeclientcount.isdigit() else None,
                ap_assocfailcountforrssilow=int(ap_assocfailcountforrssilow) if ap_assocfailcountforrssilow.isdigit() else None,
                ap_assocfailtimes=int(ap_assocfailtimes) if ap_assocfailtimes.isdigit() else None,
                ap_echorequestcount=int(ap_echorequestcount) if ap_echorequestcount.isdigit() else None,
                ap_echoresponselosscount=int(ap_echoresponselosscount) if ap_echoresponselosscount.isdigit() else None,
            )

    return None

###########################################################################
#
#  SNMP DATA Parser function
#
###########################################################################


def parse_cisco_wlc(string_table: List[StringTable]) -> Optional[Dict[str, Ap]]:
    try:
        bsnap, lwapap, cdpdata = string_table
    except ValueError:
        return

    parsed = {}

    for ap in bsnap:
        ap_oid_end, ap_dot3_mac_address, bsnap_name, ap_location, ap_operationstatus, ap_model, ap_serialnumber, \
        ap_ipaddress, ap_ethernetmacaddress = ap

        ap_info = ApInfo(
            ap_location=ap_location,
            ap_operationstatus=int(ap_operationstatus) if ap_operationstatus.isdigit else None,
            ap_model=ap_model,
            ap_serialnumber=ap_serialnumber,
            ap_ipaddress=ap_ipaddress,
            ap_ethernetmacaddress=_render_mac_address(ap_ethernetmacaddress),
            ap_radio_mac=_render_mac_address(ap_dot3_mac_address),
        )

        parsed.update({
            bsnap_name: Ap(
                ap_info=ap_info,
                cdp_info=_get_cdpdata(bsnap_name, cdpdata),
                lwap_info=_get_lwapap_info(ap_oid_end, lwapap),
            )
        })
    return parsed


###########################################################################
#
#  DISCOVERY function
#
###########################################################################


def discovery_cisco_wlc(section: Section) -> DiscoveryResult:
    for ap_name in section.keys():
        ap = section.get(ap_name)

        inv_ap_info = {
            'ap_location': ap.ap_info.ap_location,
            'ap_model': ap.ap_info.ap_model,
            'ap_serialnumber': ap.ap_info.ap_serialnumber,
            'ap_ipaddress': ap.ap_info.ap_ipaddress,
            'ap_ethernetmacaddress': ap.ap_info.ap_ethernetmacaddress,
        }

        if ap.cdp_info:
            inv_ap_info.update({
                'cdp_neigh_name': ap.cdp_info.cdp_neigh_name,
                'cdp_neigh_address': ap.cdp_info.cdp_neigh_address,
                'cdp_neigh_interface': ap.cdp_info.cdp_neigh_interface,
                'cdp_neigh_platform': ap.cdp_info.cdp_neigh_platform,
            })

        yield Service(item=ap_name, parameters={'inv_ap_info': inv_ap_info})


###########################################################################
#
#  CHECK function
#
###########################################################################


def check_cisco_wlc(item, params, section: Section) -> CheckResult:
    ap_missing_state = params.get('state_ap_missing', 1)
    if params.get('inv_ap_info'):
        inv_ap_info = InvApInfo(
            ap_location=params.get('inv_ap_info').get('ap_location'),
            ap_model=params.get('inv_ap_info').get('ap_model'),
            ap_serialnumber=params.get('inv_ap_info').get('ap_serialnumber'),
            ap_ipaddress=params.get('inv_ap_info').get('ap_ipaddress'),
            ap_ethernetmacaddress=params.get('inv_ap_info').get('ap_ethernetmacaddress'),
            cdp_neigh_name=params.get('inv_ap_info').get('cdp_neigh_name', None),
            cdp_neigh_address=params.get('inv_ap_info').get('cdp_neigh_address', None),
            cdp_neigh_interface=params.get('inv_ap_info').get('cdp_neigh_interface', None),
            cdp_neigh_platform=params.get('inv_ap_info').get('cdp_neigh_platform', None),
        )
    else:
        inv_ap_info = None

    for ap_name, ap_alias in params.get('ap_list', []):
        if item == ap_name:
            yield Result(state=State.OK, summary=f'Alias: {ap_alias}')

    try:
        ap = section[item]
    except KeyError:
        # Special treatment if this device is APs
        if inv_ap_info:
            yield Result(state=State.OK, notice=f'\nAP Missing')
            yield Result(state=State.OK, notice=f' - AP info:'
                                                f'\nLocation: {inv_ap_info.ap_location},'
                                                f'\nModel: {inv_ap_info.ap_model},'
                                                f'\nSerial: {inv_ap_info.ap_serialnumber},'
                                                f'\nIP-address: {inv_ap_info.ap_ipaddress},'
                                                f'\nMAC-address: {inv_ap_info.ap_ethernetmacaddress}'
                         )
            if inv_ap_info.cdp_neigh_name:
                yield Result(state=State.OK, notice=f' - CDP Info:'
                                                    f'\nNeighbour: {inv_ap_info.cdp_neigh_name},'
                                                    f'\nInterface: {inv_ap_info.cdp_neigh_interface},'
                                                    f'\nPlatform: {inv_ap_info.cdp_neigh_platform},'
                                                    f'\nIP-address: {inv_ap_info.cdp_neigh_address}'
                             )
        yield Result(
            state=State(ap_missing_state),
            summary=f'AP {item} not found in SNMP data. For more information see check details (long output)',
        )
        return

    ap_info = ap.ap_info
    cdp_info = ap.cdp_info
    lwap_info = ap.lwap_info

    if lwap_info:
        if lwap_info.ap_uptime:
            yield from check_levels(
                value=time() - (lwap_info.ap_uptime / 100),
                label='Up since',
                render_func=render.datetime,
                notice_only=True
            )
            yield from check_levels(
                value=lwap_info.ap_uptime / 100,
                label='Uptime',
                render_func=render.timespan,
                metric_name='cisco_wlc_uptime_seconds',
            )

        if lwap_info.ap_activeclientcount:
            yield Result(state=State.OK, summary=f'Clients {lwap_info.ap_activeclientcount}')

        if lwap_info.ap_lwappuptime:
            yield from check_levels(
                value=lwap_info.ap_lwappuptime / 100,
                label='Controller joined for',
                render_func=render.timespan,
                metric_name='cisco_wlc_lwap_uptime_seconds',
                notice_only=True
            )

        if lwap_info.ap_lwappjointakentime:
            yield from check_levels(
                value=lwap_info.ap_lwappjointakentime / 100,
                label='Time taken to join controller',
                render_func=render.timespan,
                notice_only=True
            )

    yield Result(state=State.OK, notice=f'\nBase data:')
    yield Result(state=State.OK, notice=f' - IP address: {ap_info.ap_ipaddress}')
    yield Result(state=State.OK, notice=f' - Ethernet MAC address: {ap_info.ap_ethernetmacaddress}')
    yield Result(state=State.OK, notice=f' - Radio MAC address: {ap_info.ap_radio_mac}')
    yield Result(state=State.OK, notice=f' - Model: {ap_info.ap_model}')
    yield Result(state=State.OK, notice=f' - S/N:  {ap_info.ap_serialnumber}')
    yield Result(state=State.OK, notice=f' - Location:  {ap_info.ap_location}')

    if cdp_info:
        yield Result(state=State.OK, notice=f'\nCDP info:')
        yield Result(state=State.OK, notice=f' - Neighbour: {cdp_info.cdp_neigh_name}')
        yield Result(state=State.OK, notice=f' - Interface: {cdp_info.cdp_neigh_interface}')
        yield Result(state=State.OK, notice=f' - Platform: {cdp_info.cdp_neigh_platform}')
        yield Result(state=State.OK, notice=f' - Address: {cdp_info.cdp_neigh_address}')

        if cdp_info.cdp_neigh_duplex == 3:
            yield Result(state=State(params.get('state_cdp_duplex', 1)), notice='CDP state is half duplex')

    yield Result(state=State.OK, notice=f'\nProblems:')

    if ap_info.ap_operationstatus is not None and (ap_info.ap_operationstatus != 1):
        operationstatus_str = _power_status.get(lwap_info.ap_powerstatus, f'unknown ({ap_info.ap_operationstatus})')
        yield Result(
            state=State(params.get('state_ap_operationstatus', 1)),
            notice=f'not operational: {operationstatus_str}',
            details=' - AP is not operational',
        )

    if lwap_info:
        if lwap_info.ap_adminstatus == 2:
            yield Result(
                state=State(params.get('state_ap_adminstatus', 1)),
                notice=f'AP is admin disabled',
                details=f' - AP is admin disabled (enable with: config ap enable {item}).',
            )

        if lwap_info.ap_telnetenable != 2:
            yield Result(
                state=State(params.get('state_ap_telnetenable', 2)),
                notice='Telent enabled',
                details=f' - Telnet enabled (disable with: config ap telnet disable {item}).'
            )

        if lwap_info.ap_sshenable != 2:
            yield Result(
                state=State(params.get('state_ap_sshenable', 1)),
                notice='SSH enabled',
                details=f' - SSH enabled (disable with: config ap ssh disable {item})'
            )

        if lwap_info.ap_encryptionsupported == 1 and lwap_info.ap_encryptionenable != 1:
            yield Result(
                state=State(params.get('stateap_encryptionenable', 1)),
                notice='encryption not enabled',
                details=f' - AP is DTLS encryption capable but DTLS tunnel encryption is not enabled '
                        f'(enable with: config ap link-encryption enable {item}).'
            )

        if lwap_info.ap_encryptionsupported == 1 \
                and lwap_info.ap_encryptionenable == 1 \
                and lwap_info.ap_dataencryptionstatus != 1:
            yield Result(
                state=State(params.get('state_ap_dataencryptionstatus', 1)),
                notice='Encryption enabled but not active',
                details=' - Encryption enabled but not active'
            )

        if lwap_info.ap_powerstatus < 4:
            powerstatus_str = _power_status.get(lwap_info.ap_powerstatus, f'unknown ({lwap_info.ap_powerstatus})')
            yield Result(
                state=State(params.get('state_ap_powerstatus', 2)),
                notice=f'Powerstatus: {powerstatus_str}',
                details=f' - AP not on full power: {powerstatus_str}'
            )

        # perfdata (plain)
        for key, value, bounderies in [
            ('active_client_count', lwap_info.ap_activeclientcount, (0, None)),
            ('associated_clients', lwap_info.ap_associatedclientcount, (0, None)),
        ]:
            if value:
                yield Metric(name=f'cisco_wlc_{key}', value=value, boundaries=bounderies)

        # perfdata (counter per second)
        now_time = time()
        value_store = get_value_store()
        rate_item = item.replace(' ', '_').replace(':', '_')
        raise_ingore_res = False

        for key, value, bounderies in [
            ('reassoiation_success_count', lwap_info.ap_reassocsucccount, (0, None)),
            ('reassoiation_failure_count', lwap_info.ap_reassocfailcount, (0, None)),
            ('association_count', lwap_info.ap_assoccount, (0, None)),
            ('association_fail_count_for_rssi_low', lwap_info.ap_assocfailcountforrssilow, (0, None)),
            ('association_fail_times', lwap_info.ap_assocfailtimes, (0, None)),
            ('echo_request_count', lwap_info.ap_echorequestcount, (0, None)),
            ('echo_response_loss_count', lwap_info.ap_echoresponselosscount, (0, None)),
        ]:
            if value is not None:
                try:
                    value = get_rate(value_store, f'cisco_wlc_.{key}.{rate_item}', now_time, value, raise_overflow=False)
                    yield Metric(name=f'cisco_wlc_{key}', value=value, boundaries=bounderies)
                except GetRateError:
                    raise_ingore_res = True

        if raise_ingore_res:
            raise IgnoreResultsError('Initializing counters')

    if cdp_info:
        if inv_ap_info.cdp_neigh_name:
            # CDP DATA Changed --> AP moved to different switch/interface
            if cdp_info.cdp_neigh_name != inv_ap_info.cdp_neigh_name or \
                    cdp_info.cdp_neigh_interface != inv_ap_info.cdp_neigh_interface or \
                    cdp_info.cdp_neigh_platform != inv_ap_info.cdp_neigh_platform or \
                    cdp_info.cdp_neigh_address != inv_ap_info.cdp_neigh_address:
                yield Result(state=State.OK, notice='\nCDP info changed. Please check ap network '
                                                    'connection or run inventory again.')
                yield Result(state=State.OK, notice=f' - CDP info found: '
                                                    f'Neighbour: {cdp_info.cdp_neigh_name}, '
                                                    f'Interface: {cdp_info.cdp_neigh_interface}, '
                                                    f'Platform: {cdp_info.cdp_neigh_platform}, '
                                                    f'Address: {cdp_info.cdp_neigh_address}')
                yield Result(state=State.OK, notice=f' - CDP info expected: '
                                                    f'Neighbour: {inv_ap_info.cdp_neigh_name}, '
                                                    f'Interface: {inv_ap_info.cdp_neigh_interface}, '
                                                    f'Platform: {inv_ap_info.cdp_neigh_platform}, '
                                                    f'Address: {inv_ap_info.cdp_neigh_address}')
                yield Result(state=State(params.get('state_cdp_change', 1)), notice='CDP info changed', )
    else:
        yield Result(state=State(params.get('state_cdp_change', 1)), notice=' - CDP info missing', )

    # ap has wrong serial number --> AP H/W changed.
    if inv_ap_info:
        if ap_info.ap_serialnumber != inv_ap_info.ap_serialnumber or \
                ap_info.ap_model != inv_ap_info.ap_model or \
                ap_info.ap_ethernetmacaddress != inv_ap_info.ap_ethernetmacaddress or \
                ap_info.ap_ipaddress != inv_ap_info.ap_ipaddress or \
                ap_info.ap_location != inv_ap_info.ap_location:
            yield Result(
                state=State(params.get('state_ap_change', 1)),
                notice='Hardware changed',
                details=f'\nAP hardware changed. Run inventory again. '
                        f'\n - Serial number (found/expected): {ap_info.ap_serialnumber} / {inv_ap_info.ap_serialnumber} '
                        f'\n - Model (found/expected): {ap_info.ap_model} / {inv_ap_info.ap_model} '
                        f'\n - MAC address (found/expected): {ap_info.ap_ethernetmacaddress} / {inv_ap_info.ap_ethernetmacaddress} '
                        f'\n - IP address (found/expected): {ap_info.ap_ipaddress} / {inv_ap_info.ap_ipaddress} '
                        f'\n - Location (found/expected): {ap_info.ap_location} / {inv_ap_info.ap_location}',
            )
    else:
        yield Result(state=State.WARN, summary='AP data from discovery missing.')

###########################################################################
#
#  Cluster function
#
###########################################################################


def _node_not_found(item: str, params: Mapping[str, Any]) -> Result:
    infotext = 'not found'
    for ap_name, ap_state in params.get('ap_name', []):
        if item.startswith(ap_name):
            return Result(state=ap_state, summary=infotext)
    return Result(state=State.CRIT, summary=infotext)


def _ap_info(node: Optional[str], wlc_status: Union[str, Ap]) -> Result:
    wlc_status = str(wlc_status.ap_info.ap_operationstatus) if isinstance(wlc_status, Ap) else wlc_status
    status, state_readable = _map_states.get(wlc_status, (State.UNKNOWN, f"unknown [{wlc_status}]"))
    return Result(state=status, summary=f'{state_readable}{f" (connected to {node})" if node else ""}',)


def cluster_check_cisco_wlc(
        item: str,
        params: Mapping[str, Any],
        section: Mapping[str, Optional[Section]],
) -> CheckResult:

    for node, node_section in section.items():
        if node_section is not None and item in node_section:
            yield _ap_info(node, node_section[item])
            yield from check_cisco_wlc(item, params, node_section)
            return
    yield _node_not_found(item, params)

###########################################################################
#
# Register SNMP section
#
###########################################################################


register.snmp_section(
    name='cisco_wlc',
    parse_function=parse_cisco_wlc,
    fetch=[
        SNMPTree(
            base='.1.3.6.1.4.1.14179.2.2.1.1',  # AIRESPACE-WIRELESS-MIB::bsnAPEntry
            oids=[
                OIDEnd(),
                '1',  # bsnAPDot3MacAddress
                '3',  # 1 bsnAPName
                '4',  # 2 bsnAPLocation
                '6',  # 3 bsnAPOperationStatus
                '16',  # 4 bsnAPModel
                '17',  # 5 bsnAPSerialNumber
                '19',  # 6 bsnApIpAddress
                '33',  # 7 bsnAPEthernetMacAddress
            ]
        ),
        SNMPTree(
            base='.1.3.6.1.4.1.9.9.513.1.1.1.1',  # CISCO-LWAPP-AP-MIB::cLApEntry
            oids=[
                OIDEnd(),
                '6',  # cLApUpTime
                '7',  # cLLwappUpTime
                '8',  # cLLwappJoinTakenTime
                '18',  # cLApEncryptionEnable
                '20',  # cLApPowerStatus
                '21',  # cLApTelnetEnable
                '22',  # cLApSshEnable
                '33',  # cLApEncryptionSupported
                '36',  # cLApDataEncryptionStatus
                '38',  # cLApAdminStatus

                '51',  # cLApAssocCount
                '54',  # cLApAssociatedClientCount
                '62',  # cLApMaxClientLimitNumberTrap
                '67',  # cLApReassocSuccCount
                '68',  # cLApReassocFailCount
                '72',  # cLApActiveClientCount
                '73',  # cLApAssocFailCountForRssiLow
                '75',  # cLApAssocFailTimes
                '78',  # cLApEchoRequestCount
                '79',  # cLApEchoResponseLossCount
            ]
        ),
        SNMPTree(
            base='.1.3.6.1.4.1.9.9.623.1.3.1.1',  # CISCO-LWAPP-CDP-MIB::clcCdpApCacheEntry
            oids=[
                '2',  # 0 clcCdpApCacheApName
                '6',  # 1 clcCdpApCacheNeighName
                '7',  # 2 clcCdpApCacheNeighAddressType
                '8',  # 3 clcCdpApCacheNeighAddress
                '9',  # 4 clcCdpApCacheNeighInterface
                '12',  # 5 clcCdpApCachePlatform
                '15',  # 6 clcCdpApCacheDuplex
            ]
        ),
    ],
    detect=any_of(
        contains('.1.3.6.1.2.1.1.1.0', 'Cisco Controller'),  # sysDescr -> AirOS
        contains('.1.3.6.1.2.1.1.1.0', 'C9800'),  # sysDescr -> IOS-XE
    ))

###########################################################################
#
# Register Check Plugin
#
###########################################################################


register.check_plugin(
    name='cisco_wlc',
    service_name='AP %s',
    discovery_function=discovery_cisco_wlc,
    check_function=check_cisco_wlc,
    check_default_parameters={
        'state_ap_missing': 1,
        'state_ap_adminstatus': 1,
        'state_ap_operationstatus': 1,
        'state_ap_telnetenable': 2,
        'state_ap_sshenable': 1,
        'stateap_encryptionenable': 1,
        'state_ap_dataencryptionstatus': 1,
        'state_ap_powerstatus': 2,
        'state_cdp_duplex': 1,
        'state_cdp_change': 1,
        'state_ap_change': 1,
        'ap_list': [],
        # 'inv_ap_info': {},
    },
    check_ruleset_name='cisco_wlc',
    cluster_check_function=cluster_check_cisco_wlc,
)
