#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  :
#
# 2023-02-21: moved from ~/local/share/check_mk/... to ~/local/lib/check_mk...
#

from cmk.gui.i18n import _
from cmk.gui.valuespec import (
    Dictionary,
    Tuple,
    TextAscii,
    ListOf,
    MonitoringState,
    TextUnicode,
)

from cmk.gui.plugins.wato.utils import (
    CheckParameterRulespecWithItem,
    rulespec_registry,
    RulespecGroupCheckParametersNetworking,
)


def _parameter_valuespec_cisco_wlc():
    return Dictionary(
        elements=[
            ('state_ap_missing',
             MonitoringState(
                 title=_('State if AP is missing'),
                 help=_('Set the monitoring state if the access point is not in the SNMP data. Default is WARN.'),
                 default_value=1,
             )),
            ('state_ap_adminstatus',
             MonitoringState(
                 title=_('State if AP is not admin enabled'),
                 help=_('Set the monitoring state if the access point is not in "admin enable". Default is WARN.'),
                 default_value=1,
             )),
            ('state_ap_operationstatus',
             MonitoringState(
                 title=_('State if AP is not operational'),
                 help=_('Set the monitoring state if the access point is not in "operational" state. Default is WARN.'),
                 default_value=1,
             )),
            ('state_ap_telnetenable',
             MonitoringState(
                 title=_('State if Telnet is enabled'),
                 help=_('Set the monitoring state if the access point has telnet access enabled. Default is CRIT.'),
                 default_value=2,
             )),
            ('state_ap_sshenable',
             MonitoringState(
                 title=_('State if SSH is enabled'),
                 help=_('Set the monitoring state if the access point has SSH access enabled. Default is WARN.'),
                 default_value=1,
             )),
            ('stateap_encryptionenable',
             MonitoringState(
                 title=_('State if encryption is not enabled'),
                 help=_('Set the monitoring state if the LEAP/CAPWAP tunnel encryption nor enabled. Default is WARN.'),
                 default_value=1,
             )),
            ('state_ap_dataencryptionstatus',
             MonitoringState(
                 title=_('State if encryption is not active'),
                 help=_('Set the monitoring state if the LWAP/CAPWAP tunnel encryption enabled but not active. '
                        'Default is WARN.'),
                 default_value=1,
             )),
            ('state_ap_powerstatus',
             MonitoringState(
                 title=_('State if AP is not on full power'),
                 help=_('Set the monitoring state if the access point not on full power. Default is CRIT.'),
                 default_value=2,
             )),
            ('state_cdp_duplex',
             MonitoringState(
                 title=_('State if CDP "half duplex"'),
                 help=_('Set the monitoring state if the the CDP neighbour info for "duplex" is "half duplex". '
                        'Default is WARN.'),
                 default_value=1,
             )),
            ('state_cdp_change',
             MonitoringState(
                 title=_('State if CDP info is changed/missing'),
                 help=_('Set the monitoring state if the CDP info from the access point has changed or is missing. '
                        'This happens if the AP was moved to a different switch/interface. The CDP info will be used '
                        'to help locating a missing access point. '
                        'Default is WARN.'),
                 default_value=1,
             )),
            ('state_ap_change',
             MonitoringState(
                 title=_('State if H/W has changed'),
                 help=_('Set the monitoring state if the hard ware of the access point has changed. This information '
                        'will be used to help locate a missing access point '
                        'Default is WARN.'),
                 default_value=1,
             )),
            ('ap_list',
             ListOf(
                 Tuple(
                     elements=[
                         TextUnicode(
                             title=_('AP name'),
                             help=_('The configured value must match a AP item as reported by the monitored '
                                    'device. For example: "AP1.4"'),
                             allow_empty=False,
                             size=40,
                         ),
                         TextUnicode(
                             title=_('AP Alias'),
                             help=_('You can configure an individual alias here for the access point matching '
                                    'the text configured in the "AP item name" field. The alias will '
                                    'be shown in the check info'),
                             allow_empty=False,
                             size=40,
                         ),
                     ],
                     orientation='horizontal',
                 ),
                 title=_('AP alias'),
                 add_label=_('Add name'))),
            ('inv_ap_info',  # added by plugin discovery function
             Dictionary(
                 elements=[
                     ('ap_location', TextUnicode()),
                     ('ap_model', TextUnicode()),
                     ('ap_serialnumber', TextUnicode()),
                     ('ap_ipaddress', TextUnicode()),
                     ('ap_ethernetmacaddress', TextUnicode()),
                     ('cdp_neigh_name', TextUnicode()),
                     ('cdp_neigh_address', TextUnicode()),
                     ('cdp_neigh_interface', TextUnicode()),
                     ('cdp_neigh_platform', TextUnicode()),
                 ],
             )),
        ],
        hidden_keys=['inv_ap_info'],
    )


rulespec_registry.register(
    CheckParameterRulespecWithItem(
        check_group_name='cisco_wlc',
        group=RulespecGroupCheckParametersNetworking,
        match_type='dict',
        parameter_valuespec=_parameter_valuespec_cisco_wlc,
        title=lambda: _('Cisco WLC APs'),
        item_spec=lambda: TextAscii(title=_('Cisco WLC AP name'), ),
    ))
